import asyncio
from typing import AsyncGenerator, Generator, TypeVar

T = TypeVar("T")

__all__ = ["make_async_generator"]


async def make_async_generator(
    sync_gen: Generator[T, None, None]
) -> AsyncGenerator[T, None]:
    for value in sync_gen:
        await asyncio.sleep(0)
        yield value



from openai.types.beta.thread import Thread

from ._base import _generic_merge_with_partial

from ..._types.partials.threads import PartialThread

__all__ = ["merge_thread_with_partial"]


def merge_thread_with_partial(t: Thread, p: PartialThread) -> Thread:
    return _generic_merge_with_partial(Thread, t, p)



import respx

from ..stores import StateStore

from .chat import ChatCompletionsCreateRoute
from .embeddings import EmbeddingsCreateRoute
from .files import (
    FileCreateRoute,
    FileListRoute,
    FileRetrieveRoute,
    FileDeleteRoute,
    FileRetrieveContentRoute,
)
from .models import ModelListRoute, ModelRetrieveRoute
from .moderation import ModerationCreateRoute

from .beta import BetaRoutes

__all__ = [
    "BetaRoutes",
    "ChatRoutes",
    "EmbeddingsRoutes",
    "FileRoutes",
    "ModelRoutes",
    "ModerationsRoutes",
]


class ChatRoutes:
    def __init__(self, router: respx.MockRouter) -> None:
        self.completions = ChatCompletionRoutes(router)


class ChatCompletionRoutes:
    def __init__(self, router: respx.MockRouter) -> None:
        self.create = ChatCompletionsCreateRoute(router)


class EmbeddingsRoutes:
    def __init__(self, router: respx.MockRouter) -> None:
        self.create = EmbeddingsCreateRoute(router)


class FileRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.create = FileCreateRoute(router, state)
        self.list = FileListRoute(router, state)
        self.retrieve = FileRetrieveRoute(router, state)
        self.delete = FileDeleteRoute(router, state)
        self.content = FileRetrieveContentRoute(router, state)


class ModelRoutes:
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        self.list = ModelListRoute(router, state)
        self.retrieve = ModelRetrieveRoute(router, state)


class ModerationsRoutes:
    def __init__(self, router: respx.MockRouter) -> None:
        self.create = ModerationCreateRoute(router)



import sys
from typing import Any, Optional
from typing_extensions import override

import httpx
import respx
from requests_toolbelt.multipart import decoder

from openai.pagination import SyncPage
from openai.types.file_object import FileObject
from openai.types.file_deleted import FileDeleted

from ._base import StatefulRoute

from ..stores import StateStore
from .._types.partials.files import (
    PartialFileObject,
    PartialFileList,
    PartialFileDeleted,
)

from .._utils.faker import faker
from .._utils.serde import model_dict
from .._utils.time import utcnow_unix_timestamp_s


__all__ = ["FileCreateRoute", "FileListRoute", "FileRetrieveRoute", "FileDeleteRoute"]


class FileCreateRoute(StatefulRoute[FileObject, PartialFileObject]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(url__regex="/files"),
            status_code=201,
            state=state,
        )

    @override
    def _handler(self, request: httpx.Request, route: respx.Route) -> httpx.Response:
        self._route = route

        filename: Optional[str] = None
        purpose: Optional[str] = None
        file_content: Optional[bytes] = None

        multipart_data = decoder.MultipartDecoder(
            request.content, request.headers.get("content-type")
        )
        for part in multipart_data.parts:  # type: ignore
            content_disposition = part.headers.get(b"Content-Disposition", b"").decode()  # type: ignore
            if 'name="purpose"' in content_disposition:
                purpose = part.text  # type: ignore
            elif 'name="file"' in content_disposition:
                filename = (  # type: ignore
                    part.headers.get(b"Content-Disposition", b"")  # type: ignore
                    .decode()
                    .split("filename=")[1]
                    .strip('"')
                )
                file_content = part.content  # type: ignore

        assert filename
        assert purpose
        assert file_content

        model = FileObject(
            id=faker.file.id(),
            bytes=sys.getsizeof(file_content),  # type: ignore
            created_at=utcnow_unix_timestamp_s(),
            filename=filename,  # type: ignore
            object="file",
            purpose=purpose,  # type: ignore
            status="uploaded",
            status_details=None,
        )
        self._state.files.put(model)
        self._state.files.content.put(model.id, file_content)  # type: ignore
        return httpx.Response(
            status_code=self._status_code,
            json=model_dict(model),
        )

    @staticmethod
    def _build(partial: PartialFileObject, request: httpx.Request) -> FileObject:
        raise NotImplementedError


class FileListRoute(StatefulRoute[SyncPage[FileObject], PartialFileList]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(url__regex="/files"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(self, request: httpx.Request, route: respx.Route) -> httpx.Response:
        self._route = route
        purpose = request.url.params.get("purpose")
        files = SyncPage[FileObject](
            object="list",
            data=self._state.files.list(purpose=purpose),
        )
        return httpx.Response(status_code=200, json=model_dict(files))

    @staticmethod
    def _build(
        partial: PartialFileList,
        request: httpx.Request,
    ) -> SyncPage[FileObject]:
        raise NotImplementedError


class FileRetrieveRoute(StatefulRoute[FileObject, PartialFileObject]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(url__regex=r"/files/(?P<file_id>[a-zA-Z0-9\-]+)"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        fil_id = kwargs["file_id"]
        found = self._state.files.get(fil_id)
        if not found:
            return httpx.Response(404)

        return httpx.Response(status_code=200, json=model_dict(found))

    @staticmethod
    def _build(
        partial: PartialFileObject,
        request: httpx.Request,
    ) -> FileObject:
        raise NotImplementedError


class FileDeleteRoute(StatefulRoute[FileObject, PartialFileDeleted]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.delete(url__regex=r"/files/(?P<file_id>[a-zA-Z0-9\-]+)"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        file_id = kwargs["file_id"]
        deleted = self._state.files.delete(file_id)
        return httpx.Response(
            status_code=200,
            json=model_dict(FileDeleted(id=file_id, deleted=deleted, object="file")),
        )

    @staticmethod
    def _build(
        partial: PartialFileDeleted,
        request: httpx.Request,
    ) -> FileObject:
        raise NotImplementedError


class FileRetrieveContentRoute(StatefulRoute[FileObject, PartialFileObject]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(url__regex=r"/files/(?P<file_id>[a-zA-Z0-9\-]+)/content"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        fil_id = kwargs["file_id"]
        found = self._state.files.get(fil_id)
        if not found:
            return httpx.Response(404)

        content = self._state.files.content.get(found.id)
        assert content
        return httpx.Response(status_code=200, content=content)

    @staticmethod
    def _build(partial: PartialFileObject, request: httpx.Request) -> FileObject:
        raise NotImplementedError



from typing import Any, Dict, List, Literal, TypedDict, Union
from typing_extensions import NotRequired

from .assistants import PartialAssistantResponseFormat, PartialAssistantTool

__all__ = ["PartialRun"]


class PartialIncompleteDetails(TypedDict):
    reason: NotRequired[Literal["max_completion_tokens", "max_prompt_tokens"]]


class PartialLastError(TypedDict):
    code: Literal["server_error", "rate_limit_exceeded", "invalid_prompt"]
    message: str


class PartialFunction(TypedDict):
    arguments: str
    name: str


class PartialRequiredActionFunctionToolCall(TypedDict):
    id: str
    function: PartialFunction
    type: Literal["function"]


class PartialRequiredActionSubmitToolOutputs(TypedDict):
    tool_calls: List[PartialRequiredActionFunctionToolCall]


class PartialRequiredAction(TypedDict):
    submit_tool_outputs: PartialRequiredActionSubmitToolOutputs
    type: Literal["submit_tool_outputs"]


class PartialAssistantToolChoiceFunction(TypedDict):
    name: str


class PartialAssistantToolChoice(TypedDict):
    type: Literal["function", "code_interpreter", "file_search"]
    function: NotRequired[PartialAssistantToolChoiceFunction]


class PartialCodeInterpreterTool(TypedDict):
    type: Literal["code_interpreter"]


class PartialFileSearchTool(TypedDict):
    type: Literal["file_search"]


class PartialFunctionDefinition(TypedDict):
    name: str
    description: NotRequired[str]
    parameters: NotRequired[Dict[str, Any]]


class PartialFunctionTool(TypedDict):
    function: PartialFunctionDefinition
    type: Literal["function"]


class PartialTruncationStrategy(TypedDict):
    type: Literal["auto", "last_messages"]
    last_messages: NotRequired[int]


class PartialUsage(TypedDict):
    completion_tokens: int
    prompt_tokens: int
    total_tokens: int


class PartialRun(TypedDict):
    id: NotRequired[str]
    assistant_id: NotRequired[str]
    cancelled_at: NotRequired[int]
    completed_at: NotRequired[int]
    created_at: NotRequired[int]
    expires_at: NotRequired[int]
    failed_at: NotRequired[int]
    incomplete_details: NotRequired[PartialIncompleteDetails]
    instructions: NotRequired[str]
    last_error: NotRequired[PartialLastError]
    max_completion_tokens: NotRequired[int]
    max_prompt_tokens: NotRequired[int]
    metadata: NotRequired[Dict[str, str]]
    model: NotRequired[str]
    object: NotRequired[Literal["thread.run"]]
    parallel_tool_calls: NotRequired[bool]
    required_action: NotRequired[PartialRequiredAction]
    response_format: NotRequired[PartialAssistantResponseFormat]
    started_at: NotRequired[int]
    status: NotRequired[
        Literal[
            "queued",
            "in_progress",
            "requires_action",
            "cancelling",
            "cancelled",
            "failed",
            "completed",
            "expired",
        ]
    ]
    thread_id: NotRequired[str]
    tool_choice: NotRequired[
        Union[Literal["none", "auto", "required"], PartialAssistantToolChoice]
    ]
    tools: NotRequired[List[PartialAssistantTool]]
    truncation_strategy: NotRequired[PartialTruncationStrategy]
    usage: NotRequired[PartialUsage]
    temperature: NotRequired[float]
    top_p: NotRequired[float]



import json
from typing import Any
from typing_extensions import override

import httpx
import respx

from openai.pagination import SyncCursorPage
from openai.types.beta.threads.run import Run
from openai.types.beta.threads.run_create_params import RunCreateParams
from openai.types.beta.threads.run_update_params import RunUpdateParams
from openai.types.beta.thread_create_and_run_params import ThreadCreateAndRunParams

from .._base import StatefulRoute

from ...helpers.builders.messages import message_from_create_request
from ...helpers.builders.threads import thread_from_create_request

from ...stores import StateStore
from ..._types.partials.sync_cursor_page import PartialSyncCursorPage
from ..._types.partials.runs import PartialRun

from ..._utils.copy import model_copy
from ..._utils.faker import faker
from ..._utils.serde import json_loads, model_dict, model_parse
from ..._utils.time import utcnow_unix_timestamp_s


__all__ = [
    "RunCreateRoute",
    "ThreadCreateAndRun",
    "RunListRoute",
    "RunRetrieveRoute",
    "RunUpdateRoute",
    "RunSubmitToolOutputsRoute",
    "RunCancelRoute",
]


class RunCreateRoute(StatefulRoute[Run, PartialRun]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/runs"
            ),
            status_code=201,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        content: RunCreateParams = json_loads(request.content)

        found_asst = self._state.beta.assistants.get(content["assistant_id"])
        if not found_asst:
            return httpx.Response(404)

        model = self._build(
            {
                "thread_id": thread_id,
                "instructions": found_asst.instructions or "",
                "model": found_asst.model,
                "tools": [model_dict(t) for t in (found_asst.tools or [])],  # type: ignore
            },
            request,
        )
        self._state.beta.threads.runs.put(model)
        return httpx.Response(
            status_code=self._status_code,
            json=model_dict(model),
        )

    @staticmethod
    def _build(partial: PartialRun, request: httpx.Request) -> Run:
        content = json_loads(request.content)
        defaults: PartialRun = {
            "id": faker.beta.thread.run.id(),
            "created_at": utcnow_unix_timestamp_s(),
            "instructions": "",
            "object": "thread.run",
            "parallel_tool_calls": True,
            "status": "queued",
        }
        return model_parse(Run, defaults | partial | content)


class ThreadCreateAndRun(StatefulRoute[Run, PartialRun]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(url__regex="/threads/runs"),
            status_code=201,
            state=state,
        )

    @override
    def _handler(self, request: httpx.Request, route: respx.Route) -> httpx.Response:
        self._route = route

        content: ThreadCreateAndRunParams = json_loads(request.content)

        found_asst = self._state.beta.assistants.get(content["assistant_id"])
        if not found_asst:
            return httpx.Response(404)

        thread_create_params = content.get("thread", {})
        encoded = json.dumps(thread_create_params).encode("utf-8")
        thread_create_req = httpx.Request("", "", content=encoded)
        thread = thread_from_create_request(thread_create_req)
        self._state.beta.threads.put(thread)

        for message_create_params in thread_create_params.get("messages", []):
            encoded = json.dumps(message_create_params).encode("utf-8")
            create_message_req = httpx.Request(method="", url="", content=encoded)
            message = message_from_create_request(thread.id, create_message_req)
            self._state.beta.threads.messages.put(message)

        model = self._build(
            {
                "thread_id": thread.id,
                "instructions": found_asst.instructions or "",
                "model": found_asst.model,
                "tools": [model_dict(t) for t in (found_asst.tools or [])],  # type: ignore
            },
            request,
        )
        self._state.beta.threads.runs.put(model)
        return httpx.Response(
            status_code=self._status_code,
            json=model_dict(model),
        )

    @staticmethod
    def _build(partial: PartialRun, request: httpx.Request) -> Run:
        content = json_loads(request.content)
        if content.get("thread"):
            del content["thread"]

        return RunCreateRoute._build(partial, request)


class RunListRoute(
    StatefulRoute[SyncCursorPage[Run], PartialSyncCursorPage[PartialRun]]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/runs"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        limit = request.url.params.get("limit")
        order = request.url.params.get("order")
        after = request.url.params.get("after")
        before = request.url.params.get("before")

        data = self._state.beta.threads.runs.list(
            thread_id,
            limit,
            order,
            after,
            before,
        )
        result_count = len(data)
        total_count = len(self._state.beta.threads.runs.list(thread_id))
        has_data = bool(result_count)
        has_more = total_count != result_count
        first_id = data[0].id if has_data else None
        last_id = data[-1].id if has_data else None
        model = SyncCursorPage[Run](data=data)
        return httpx.Response(
            status_code=200,
            json=model_dict(model)
            | {"first_id": first_id, "last_id": last_id, "has_more": has_more},
        )

    @staticmethod
    def _build(
        partial: PartialSyncCursorPage[PartialRun],
        request: httpx.Request,
    ) -> SyncCursorPage[Run]:
        raise NotImplementedError


class RunRetrieveRoute(StatefulRoute[Run, PartialRun]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/runs/(?P<run_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        run_id = kwargs["run_id"]
        found_run = self._state.beta.threads.runs.get(run_id)
        if not found_run:
            return httpx.Response(404)

        return httpx.Response(status_code=200, json=model_dict(found_run))

    @staticmethod
    def _build(partial: PartialRun, request: httpx.Request) -> Run:
        raise NotImplementedError


class RunUpdateRoute(StatefulRoute[Run, PartialRun]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/runs/(?P<run_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        run_id = kwargs["run_id"]
        found_run = self._state.beta.threads.runs.get(run_id)
        if not found_run:
            return httpx.Response(404)

        content: RunUpdateParams = json_loads(request.content)
        deserialized = model_dict(found_run)
        updated = model_parse(Run, deserialized | content)
        self._state.beta.threads.runs.put(updated)

        return httpx.Response(status_code=200, json=model_dict(updated))

    @staticmethod
    def _build(partial: PartialRun, request: httpx.Request) -> Run:
        raise NotImplementedError


class RunSubmitToolOutputsRoute(StatefulRoute[Run, PartialRun]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/runs/(?P<run_id>[a-zA-Z0-9\_]+)/submit_tool_outputs"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        # TODO: update associated run step in store
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        run_id = kwargs["run_id"]
        found_run = self._state.beta.threads.runs.get(run_id)
        if not found_run:
            return httpx.Response(404)

        return httpx.Response(status_code=200, json=model_dict(found_run))

    @staticmethod
    def _build(partial: PartialRun, request: httpx.Request) -> Run:
        raise NotImplementedError


class RunCancelRoute(StatefulRoute[Run, PartialRun]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/runs/(?P<run_id>[a-zA-Z0-9\_]+)/cancel"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        # TODO: update associated run step in store
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        run_id = kwargs["run_id"]
        found_run = self._state.beta.threads.runs.get(run_id)
        if not found_run:
            return httpx.Response(404)

        found_run.status = "cancelled"
        self._state.beta.threads.runs.put(found_run)
        copy = model_copy(found_run)
        copy.status = "cancelling"

        return httpx.Response(status_code=200, json=model_dict(copy))

    @staticmethod
    def _build(partial: PartialRun, request: httpx.Request) -> Run:
        raise NotImplementedError



from typing import Dict, List, Literal, TypedDict
from typing_extensions import NotRequired

__all__ = ["PartialThread"]


class PartialToolResourcesCodeInterpreter(TypedDict):
    file_ids: NotRequired[List[str]]


class PartialToolResourcesFileSearch(TypedDict):
    vector_store_ids: NotRequired[List[str]]


class PartialToolResources(TypedDict):
    code_interpreter: NotRequired[PartialToolResourcesCodeInterpreter]
    file_search: NotRequired[PartialToolResourcesFileSearch]


class PartialThread(TypedDict):
    id: NotRequired[str]
    created_at: NotRequired[int]
    tool_resources: NotRequired[PartialToolResources]
    metadata: NotRequired[Dict[str, str]]
    object: NotRequired[Literal["thread"]]



from typing import Optional

import httpx

from openai.types.beta.threads.run import Run

from ._base import _generic_builder
from ..._routes.beta.runs import RunCreateRoute
from ..._types.partials.runs import PartialRun

__all__ = ["run_from_create_request"]


def run_from_create_request(
    thread_id: str,
    request: httpx.Request,
    *,
    extra: Optional[PartialRun] = None,
) -> Run:
    partial: PartialRun = {"thread_id": thread_id}
    if extra:
        partial |= extra
    return _generic_builder(RunCreateRoute, request, partial)



from typing import List, Literal, TypedDict
from typing_extensions import NotRequired

__all__ = ["PartialCreateEmbeddingResponse"]


class PartialEmbedding(TypedDict):
    embedding: List[float]
    index: int
    object: Literal["embedding"]


class PartialUsage(TypedDict):
    prompt_tokens: int
    total_tokens: int


class PartialCreateEmbeddingResponse(TypedDict):
    data: NotRequired[List[PartialEmbedding]]
    model: NotRequired[str]
    object: NotRequired[Literal["list"]]
    usage: NotRequired[PartialUsage]



from typing import Optional, Type

import httpx

from ..._routes._base import Route
from ..._types.generics import M, P

__all__ = ["_generic_builder"]


def _generic_builder(
    route: Type[Route[M, P]],
    request: httpx.Request,
    extra: Optional[P] = None,
) -> M:
    return getattr(route, "_build")(extra or {}, request)



from typing import Any, Mapping, Type
from ..._types.generics import M
from ..._utils.serde import model_dict, model_parse

__all__ = ["_generic_merge_with_partial"]


def _generic_merge_with_partial(t: Type[M], m: M, p: Mapping[str, Any]) -> M:
    merged = model_dict(m) | dict(p)
    return model_parse(t, merged)



from typing import Any, Literal
from typing_extensions import override

import httpx
import respx

from openai.pagination import SyncCursorPage
from openai.types.beta.vector_stores.vector_store_file import VectorStoreFile
from openai.types.beta.vector_stores.vector_store_file_deleted import (
    VectorStoreFileDeleted,
)

from .._base import StatefulRoute

from ...stores import StateStore
from ..._types.partials.deleted import PartialResourceDeleted
from ..._types.partials.sync_cursor_page import PartialSyncCursorPage
from ..._types.partials.vector_store_files import PartialVectorStoreFile

from ..._utils.serde import json_loads, model_dict, model_parse
from ..._utils.time import utcnow_unix_timestamp_s

__all__ = [
    "VectorStoreFileCreateRoute",
    "VectorStoreFileListRoute",
    "VectorStoreFileRetrieveRoute",
    "VectorStoreFileDeleteRoute",
]


class VectorStoreFileCreateRoute(
    StatefulRoute[VectorStoreFile, PartialVectorStoreFile]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)/files"
            ),
            status_code=201,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        vector_store_id = kwargs["vector_store_id"]
        found_vector_store = self._state.beta.vector_stores.get(vector_store_id)
        if not found_vector_store:
            return httpx.Response(404)

        model = self._build({"vector_store_id": vector_store_id}, request)
        found_file = self._state.files.get(model.id)
        if not found_file:
            return httpx.Response(404)

        self._state.beta.vector_stores.files.put(model)
        return httpx.Response(status_code=self._status_code, json=model_dict(model))

    @staticmethod
    def _build(
        partial: PartialVectorStoreFile,
        request: httpx.Request,
    ) -> VectorStoreFile:
        content = json_loads(request.content)
        defaults: PartialVectorStoreFile = {
            "id": content["file_id"],
            "created_at": utcnow_unix_timestamp_s(),
            "object": "vector_store.file",
            "status": "completed",
            "usage_bytes": 0,
        }
        return model_parse(VectorStoreFile, defaults | partial)


class VectorStoreFileListRoute(
    StatefulRoute[
        SyncCursorPage[VectorStoreFile], PartialSyncCursorPage[PartialVectorStoreFile]
    ]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)/files"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        vector_store_id = kwargs["vector_store_id"]
        found_vector_store = self._state.beta.vector_stores.get(vector_store_id)
        if not found_vector_store:
            return httpx.Response(404)

        limit = request.url.params.get("limit")
        order = request.url.params.get("order")
        after = request.url.params.get("after")
        before = request.url.params.get("before")
        filter = request.url.params.get("filter")

        data = self._state.beta.vector_stores.files.list(
            vector_store_id,
            limit,
            order,
            after,
            before,
            filter,
        )
        result_count = len(data)
        total_count = len(self._state.beta.vector_stores.files.list(vector_store_id))
        has_data = bool(result_count)
        has_more = total_count != result_count
        first_id = data[0].id if has_data else None
        last_id = data[-1].id if has_data else None
        model = SyncCursorPage[VectorStoreFile](data=data)
        return httpx.Response(
            status_code=200,
            json=model_dict(model)
            | {"first_id": first_id, "last_id": last_id, "has_more": has_more},
        )

    @staticmethod
    def _build(
        partial: PartialSyncCursorPage[PartialVectorStoreFile],
        request: httpx.Request,
    ) -> SyncCursorPage[VectorStoreFile]:
        raise NotImplementedError


class VectorStoreFileRetrieveRoute(
    StatefulRoute[VectorStoreFile, PartialVectorStoreFile]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)/files/(?P<file_id>[a-zA-Z0-9\-]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        vector_store_id = kwargs["vector_store_id"]
        found_vector_store = self._state.beta.vector_stores.get(vector_store_id)
        if not found_vector_store:
            return httpx.Response(404)

        file_id = kwargs["file_id"]
        found = self._state.beta.vector_stores.files.get(file_id)
        if not found:
            return httpx.Response(404)

        return httpx.Response(status_code=self._status_code, json=model_dict(found))

    @staticmethod
    def _build(
        partial: PartialVectorStoreFile,
        request: httpx.Request,
    ) -> VectorStoreFile:
        raise NotImplementedError


class VectorStoreFileDeleteRoute(
    StatefulRoute[
        VectorStoreFileDeleted,
        PartialResourceDeleted[Literal["vector_store.file.deleted"]],
    ]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.delete(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)/files/(?P<file_id>[a-zA-Z0-9\-]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        vector_store_id = kwargs["vector_store_id"]
        found_vector_store = self._state.beta.vector_stores.get(vector_store_id)
        if not found_vector_store:
            return httpx.Response(404)

        file_id = kwargs["file_id"]
        found = self._state.beta.vector_stores.files.get(file_id)
        if not found:
            return httpx.Response(404)

        deleted = self._state.beta.vector_stores.files.delete(file_id)

        return httpx.Response(
            status_code=200,
            json=model_dict(
                VectorStoreFileDeleted(
                    id=file_id,
                    deleted=deleted,
                    object="vector_store.file.deleted",
                )
            ),
        )

    @staticmethod
    def _build(
        partial: PartialResourceDeleted[Literal["vector_store.file.deleted"]],
        request: httpx.Request,
    ) -> VectorStoreFileDeleted:
        raise NotImplementedError



import httpx
import respx

from openai.types.moderation import (
    Moderation,
    Categories,
    CategoryScores,
    CategoryAppliedInputTypes,
)
from openai.types.moderation_create_response import ModerationCreateResponse

from ._base import StatelessRoute

from .._types.partials.moderation import (
    PartialModerationCreateResponse,
    PartialCategories,
    PartialCategoryScores,
    PartialCategoryAppliedInputTypes,
)

from .._utils.serde import model_parse
from .._utils.faker import faker

__all__ = ["ModerationCreateRoute"]

_default_categories: PartialCategories = {
    "harassment": False,
    "harassment/threatening": False,
    "hate": False,
    "hate/threatening": False,
    "illicit": False,
    "illicit/violent": False,
    "self-harm": False,
    "self-harm/instructions": False,
    "self-harm/intent": False,
    "sexual": False,
    "sexual/minors": False,
    "violence": False,
    "violence/graphic": False,
}

_default_category_scores: PartialCategoryScores = {
    "harassment": 0.0,
    "harassment/threatening": 0.0,
    "hate": 0.0,
    "hate/threatening": 0.0,
    "illicit": 0.0,
    "illicit/violent": 0.0,
    "self-harm": 0.0,
    "self-harm/instructions": 0.0,
    "self-harm/intent": 0.0,
    "sexual": 0.0,
    "sexual/minors": 0.0,
    "violence": 0.0,
    "violence/graphic": 0.0,
}

_default_category_applied_input_types: PartialCategoryAppliedInputTypes = {
    "harassment": [],
    "harassment/threatening": [],
    "hate": [],
    "hate/threatening": [],
    "illicit": [],
    "illicit/violent": [],
    "self-harm": [],
    "self-harm/instructions": [],
    "self-harm/intent": [],
    "sexual": [],
    "sexual/minors": [],
    "violence": [],
    "violence/graphic": [],
}


class ModerationCreateRoute(
    StatelessRoute[ModerationCreateResponse, PartialModerationCreateResponse]
):
    def __init__(self, router: respx.MockRouter) -> None:
        super().__init__(route=router.post(url__regex="/moderations"), status_code=200)

    @staticmethod
    def _build(
        partial: PartialModerationCreateResponse, request: httpx.Request
    ) -> ModerationCreateResponse:
        partial_results = partial.get("results", [])
        moderation_results = [
            Moderation(
                categories=model_parse(
                    Categories,
                    _default_categories
                    | partial_result.get("categories", _default_categories),
                ),
                category_applied_input_types=model_parse(
                    CategoryAppliedInputTypes,
                    _default_category_applied_input_types,
                ),
                category_scores=model_parse(
                    CategoryScores,
                    _default_category_scores
                    | partial_result.get("category_scores", _default_category_scores),
                ),
                flagged=partial_result.get("flagged", False),
            )
            for partial_result in partial_results
        ]

        return ModerationCreateResponse(
            id=partial.get("id", faker.moderation.id()),
            model=partial.get("model", "text-moderation-007"),
            results=moderation_results,
        )



from typing import Any, Dict

import httpx
import respx

from openai.types.chat.parsed_chat_completion import ParsedChatCompletion

from .._base import StatelessRoute

from ..._types.partials.chat import PartialParsedChatCompletion

from ..._utils.faker import faker
from ..._utils.serde import json_loads, model_parse
from ..._utils.time import utcnow_unix_timestamp_s

__all__ = ["ParsedChatCompletionsCreateRoute"]


class ParsedChatCompletionsCreateRoute(
    StatelessRoute[ParsedChatCompletion[Dict[str, Any]], PartialParsedChatCompletion]
):
    def __init__(self, router: respx.MockRouter) -> None:
        super().__init__(
            route=router.post(url__regex="/chat/completions"),
            status_code=201,
        )

    @staticmethod
    def _build(
        partial: PartialParsedChatCompletion,
        request: httpx.Request,
    ) -> ParsedChatCompletion[Dict[str, Any]]:
        content = json_loads(request.content)
        defaults: PartialParsedChatCompletion = {
            "id": partial.get("id", faker.chat.completion.id()),
            "created": partial.get("created", utcnow_unix_timestamp_s()),
            "object": "chat.completion",
        }
        return model_parse(ParsedChatCompletion[Any], defaults | partial | content)



from typing import Any, Dict, List, Literal, TypedDict
from typing_extensions import NotRequired

__all__ = ["PartialChatCompletion", "PartialParsedChatCompletion"]


class PartialFunctionCall(TypedDict):
    arguments: str
    name: str


class PartialToolCall(TypedDict):
    id: str
    function: PartialFunctionCall
    type: Literal["function"]


class PartialChatCompletionMessage(TypedDict):
    content: NotRequired[str]
    refusal: NotRequired[str]
    role: Literal["assistant"]
    function_call: NotRequired[PartialFunctionCall]
    tool_calls: NotRequired[List[PartialToolCall]]


class PartialTopLogprob(TypedDict):
    token: str
    bytes: NotRequired[List[int]]
    logprob: float


class PartialChatCompletionTokenLogprob(TypedDict):
    token: str
    bytes: NotRequired[List[int]]
    logprob: float
    top_logprobs: List[PartialTopLogprob]


class PartialChoiceLogprobs(TypedDict):
    content: NotRequired[List[PartialChatCompletionTokenLogprob]]


class PartialChoice(TypedDict):
    finish_reason: Literal[
        "stop",
        "length",
        "tool_calls",
        "content_filter",
        "function_call",
    ]
    index: int
    logprops: NotRequired[PartialChatCompletionTokenLogprob]
    message: PartialChatCompletionMessage


class PartialCompletionUsage(TypedDict):
    completion_tokens: int
    prompt_tokens: int
    total_tokens: int


class PartialChatCompletion(TypedDict):
    id: NotRequired[str]
    choices: NotRequired[List[PartialChoice]]
    created: NotRequired[int]
    model: NotRequired[str]
    object: NotRequired[Literal["chat.completion"]]
    service_tier: NotRequired[Literal["scale", "default"]]
    system_fingerprint: NotRequired[str]
    usage: NotRequired[PartialCompletionUsage]


class PartialParsedChatCompletionMessage(PartialChatCompletionMessage):
    parsed: NotRequired[Dict[str, Any]]


class PartialParsedChoice(PartialChoice):
    message: PartialParsedChatCompletionMessage  # type: ignore


class PartialParsedChatCompletion(PartialChatCompletion):
    choices: NotRequired[List[PartialParsedChoice]]  # type: ignore



__all__ = ["SYSTEM_MODELS"]

SYSTEM_MODELS = [
    {
        "id": "gpt-4-1106-preview",
        "object": "model",
        "created": 1698957206,
        "owned_by": "system",
    },
    {
        "id": "tts-1-1106",
        "object": "model",
        "created": 1699053241,
        "owned_by": "system",
    },
    {
        "id": "dall-e-2",
        "object": "model",
        "created": 1698798177,
        "owned_by": "system",
    },
    {
        "id": "tts-1",
        "object": "model",
        "created": 1681940951,
        "owned_by": "openai-internal",
    },
    {
        "id": "tts-1-hd-1106",
        "object": "model",
        "created": 1699053533,
        "owned_by": "system",
    },
    {
        "id": "tts-1-hd",
        "object": "model",
        "created": 1699046015,
        "owned_by": "system",
    },
    {
        "id": "dall-e-3",
        "object": "model",
        "created": 1698785189,
        "owned_by": "system",
    },
    {
        "id": "whisper-1",
        "object": "model",
        "created": 1677532384,
        "owned_by": "openai-internal",
    },
    {
        "id": "gpt-4o-2024-08-06",
        "object": "model",
        "created": 1722814719,
        "owned_by": "system",
    },
    {
        "id": "text-embedding-3-large",
        "object": "model",
        "created": 1705953180,
        "owned_by": "system",
    },
    {
        "id": "text-embedding-3-small",
        "object": "model",
        "created": 1705948997,
        "owned_by": "system",
    },
    {
        "id": "text-embedding-ada-002",
        "object": "model",
        "created": 1671217299,
        "owned_by": "openai-internal",
    },
    {
        "id": "o1-mini-2024-09-12",
        "object": "model",
        "created": 1725648979,
        "owned_by": "system",
    },
    {
        "id": "gpt-4-turbo",
        "object": "model",
        "created": 1712361441,
        "owned_by": "system",
    },
    {
        "id": "gpt-4-turbo-2024-04-09",
        "object": "model",
        "created": 1712601677,
        "owned_by": "system",
    },
    {
        "id": "gpt-4-0125-preview",
        "object": "model",
        "created": 1706037612,
        "owned_by": "system",
    },
    {
        "id": "gpt-3.5-turbo",
        "object": "model",
        "created": 1677610602,
        "owned_by": "openai",
    },
    {
        "id": "gpt-4o",
        "object": "model",
        "created": 1715367049,
        "owned_by": "system",
    },
    {
        "id": "gpt-4-turbo-preview",
        "object": "model",
        "created": 1706037777,
        "owned_by": "system",
    },
    {
        "id": "gpt-4o-2024-05-13",
        "object": "model",
        "created": 1715368132,
        "owned_by": "system",
    },
    {
        "id": "gpt-3.5-turbo-0125",
        "object": "model",
        "created": 1706048358,
        "owned_by": "system",
    },
    {
        "id": "o1-mini",
        "object": "model",
        "created": 1725649008,
        "owned_by": "system",
    },
    {
        "id": "gpt-4-32k",
        "object": "model",
        "created": 1687979321,
        "owned_by": "openai",
    },
    {
        "id": "o1-preview",
        "object": "model",
        "created": 1725648897,
        "owned_by": "system",
    },
    {
        "id": "gpt-3.5-turbo-1106",
        "object": "model",
        "created": 1698959748,
        "owned_by": "system",
    },
    {
        "id": "chatgpt-4o-latest",
        "object": "model",
        "created": 1723515131,
        "owned_by": "system",
    },
    {
        "id": "gpt-4-32k-0613",
        "object": "model",
        "created": 1686614047,
        "owned_by": "openai",
    },
    {
        "id": "o1-preview-2024-09-12",
        "object": "model",
        "created": 1725648865,
        "owned_by": "system",
    },
    {
        "id": "gpt-3.5-turbo-16k",
        "object": "model",
        "created": 1683758102,
        "owned_by": "openai-internal",
    },
    {
        "id": "gpt-3.5-turbo-instruct-0914",
        "object": "model",
        "created": 1694122472,
        "owned_by": "system",
    },
    {
        "id": "gpt-4",
        "object": "model",
        "created": 1687882411,
        "owned_by": "openai",
    },
    {
        "id": "gpt-4-0613",
        "object": "model",
        "created": 1686588896,
        "owned_by": "openai",
    },
    {
        "id": "gpt-3.5-turbo-instruct",
        "object": "model",
        "created": 1692901427,
        "owned_by": "system",
    },
    {
        "id": "babbage-002",
        "object": "model",
        "created": 1692634615,
        "owned_by": "system",
    },
    {
        "id": "davinci-002",
        "object": "model",
        "created": 1692634301,
        "owned_by": "system",
    },
    {
        "id": "gpt-4o-mini-2024-07-18",
        "object": "model",
        "created": 1721172717,
        "owned_by": "system",
    },
    {
        "id": "gpt-4o-mini",
        "object": "model",
        "created": 1721172741,
        "owned_by": "system",
    },
]
"""Exported from https://api.openai.com/v1/models on 2024-09-13"""



from typing import Annotated, Any, Dict, Literal, List, Optional, TypedDict, Union
from typing_extensions import NotRequired

from openai._utils._transform import PropertyInfo

__all__ = ["PartialAssistant"]


class PartialFunctionDefinition(TypedDict):
    name: str
    description: NotRequired[str]
    parameters: NotRequired[Dict[str, Any]]
    strict: NotRequired[bool]


class PartialCodeInterpreterTool(TypedDict):
    type: Literal["code_interpreter"]


class PartialFileSearch(TypedDict):
    max_num_results: NotRequired[int]


class PartialFileSearchTool(TypedDict):
    type: Literal["file_search"]
    file_search: Optional[PartialFileSearch]


class PartialFunctionTool(TypedDict):
    type: Literal["function"]
    function: PartialFunctionDefinition


class PartialResponseFormatText(TypedDict):
    type: Literal["text"]


class PartialResponseFormatJSONObject(TypedDict):
    type: Literal["json_object"]


class PartialJSONSchema(TypedDict):
    name: str
    description: NotRequired[str]
    schema: NotRequired[Dict[str, Any]]
    strict: Optional[bool]


class PartialResponseFormatJSONSchema(TypedDict):
    type: Literal["json_schema"]
    json_schema: PartialJSONSchema


class PartialToolResourcesCodeInterpreter(TypedDict):
    file_ids: NotRequired[List[str]]


class PartialToolResourcesFileSearch(TypedDict):
    vector_store_ids: NotRequired[List[str]]


class PartialToolResources(TypedDict):
    code_interpreter: NotRequired[PartialToolResourcesCodeInterpreter]
    file_search: NotRequired[PartialToolResourcesFileSearch]


PartialAssistantResponseFormat = Union[
    Literal["auto"],
    PartialResponseFormatText,
    PartialResponseFormatJSONObject,
    PartialResponseFormatJSONSchema,
]

PartialAssistantTool = Annotated[
    Union[
        PartialCodeInterpreterTool,
        PartialFileSearchTool,
        PartialFunctionTool,
    ],
    PropertyInfo(discriminator="type"),
]


class PartialAssistant(TypedDict):
    id: NotRequired[str]
    created_at: NotRequired[int]
    description: NotRequired[str]
    instructions: NotRequired[str]
    metadata: NotRequired[Dict[str, str]]
    model: NotRequired[str]
    name: NotRequired[str]
    object: NotRequired[Literal["assistant"]]
    tools: NotRequired[List[PartialAssistantTool]]
    response_format: NotRequired[PartialAssistantResponseFormat]
    temperature: NotRequired[float]
    tool_resources: NotRequired[PartialToolResources]
    top_p: NotRequired[float]



from openai.types.beta.threads.run import Run

from ._base import _generic_merge_with_partial

from ..._types.partials.runs import PartialRun

__all__ = ["merge_run_with_partial"]


def merge_run_with_partial(r: Run, p: PartialRun) -> Run:
    return _generic_merge_with_partial(Run, r, p)



import json
from typing import Any, Type

from openai import BaseModel

from .._types.generics import M

__all__ = ["json_loads", "model_dict", "model_parse"]


def json_loads(b: bytes) -> Any:
    d = json.loads(b)
    return {k: v for k, v in d.items() if v is not None}


def model_dict(m: BaseModel, **kwargs: Any) -> dict[str, Any]:
    if hasattr(m, "model_dump"):
        return getattr(m, "model_dump")(**kwargs)
    else:
        return getattr(m, "dict")(**kwargs)


def model_parse(m: Type[M], d: object, **kwargs: Any) -> M:
    if hasattr(m, "model_validate"):
        return getattr(m, "model_validate")(d, **kwargs)
    else:
        return getattr(m, "parse_obj")(d, **kwargs)



from typing import Literal, TypedDict
from typing_extensions import NotRequired


class PartialModel(TypedDict):
    id: NotRequired[str]
    created: NotRequired[int]
    object: NotRequired[Literal["model"]]
    owned_by: NotRequired[str]



import json
from typing import Any
from typing_extensions import override

import httpx
import respx

from openai.pagination import SyncCursorPage
from openai.types.beta.vector_store import VectorStore
from openai.types.beta.vector_store_create_params import VectorStoreCreateParams
from openai.types.beta.vector_store_update_params import VectorStoreUpdateParams
from openai.types.beta.vector_store_deleted import VectorStoreDeleted

from .._base import StatefulRoute

from ...helpers.builders.vector_store_files import vector_store_file_from_create_request

from ...stores import StateStore
from ..._types.partials.sync_cursor_page import PartialSyncCursorPage
from ..._types.partials.vector_stores import (
    PartialVectorStore,
    PartialVectorStoreDeleted,
)

from ..._utils.faker import faker
from ..._utils.serde import json_loads, model_dict, model_parse
from ..._utils.time import utcnow_unix_timestamp_s

__all__ = [
    "VectorStoreCreateRoute",
    "VectorStoreListRoute",
    "VectorStoreRetrieveRoute",
    "VectorStoreUpdateRoute",
    "VectorStoreDeleteRoute",
]


class VectorStoreCreateRoute(StatefulRoute[VectorStore, PartialVectorStore]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(url__regex="/vector_stores"),
            status_code=201,
            state=state,
        )

    @override
    def _handler(self, request: httpx.Request, route: respx.Route) -> httpx.Response:
        self._route = route
        model = self._build({}, request)
        self._state.beta.vector_stores.put(model)
        content: VectorStoreCreateParams = json_loads(request.content)
        file_ids = content.get("file_ids", [])
        for file_id in file_ids:
            found_file = self._state.files.get(file_id)
            if not found_file:
                return httpx.Response(404)
            encoded = json.dumps({"file_id": found_file.id})
            create_file_req = httpx.Request(method="", url="", content=encoded)
            vector_store_file = vector_store_file_from_create_request(
                create_file_req,
                extra={"vector_store_id": model.id},
            )
            self._state.beta.vector_stores.files.put(vector_store_file)

        return httpx.Response(status_code=self._status_code, json=model_dict(model))

    @staticmethod
    def _build(partial: PartialVectorStore, request: httpx.Request) -> VectorStore:
        content = json_loads(request.content)
        defaults: PartialVectorStore = {
            "id": faker.beta.vector_store.id(),
            "created_at": utcnow_unix_timestamp_s(),
            "file_counts": {
                "cancelled": 0,
                "completed": len(content.get("file_ids", [])),
                "failed": 0,
                "in_progress": 0,
                "total": 0,
            },
            "name": "",
            "object": "vector_store",
            "status": "completed",
            "usage_bytes": 0,
        }
        if content.get("file_ids"):
            del content["file_ids"]
        return model_parse(VectorStore, defaults | partial | content)


class VectorStoreListRoute(
    StatefulRoute[
        SyncCursorPage[VectorStore], PartialSyncCursorPage[PartialVectorStore]
    ]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(url__regex="/vector_stores"),
            status_code=200,
            state=state,
        )

    @override
    def _handler(self, request: httpx.Request, route: respx.Route) -> httpx.Response:
        self._route = route

        limit = request.url.params.get("limit")
        order = request.url.params.get("order")
        after = request.url.params.get("after")
        before = request.url.params.get("before")

        data = self._state.beta.vector_stores.list(limit, order, after, before)
        result_count = len(data)
        total_count = len(self._state.beta.vector_stores.list())
        has_data = bool(result_count)
        has_more = total_count != result_count
        first_id = data[0].id if has_data else None
        last_id = data[-1].id if has_data else None
        model = SyncCursorPage[VectorStore](data=data)
        return httpx.Response(
            status_code=200,
            json=model_dict(model)
            | {"first_id": first_id, "last_id": last_id, "has_more": has_more},
        )

    @staticmethod
    def _build(
        partial: PartialSyncCursorPage[PartialVectorStore],
        request: httpx.Request,
    ) -> SyncCursorPage[VectorStore]:
        raise NotImplementedError


class VectorStoreRetrieveRoute(StatefulRoute[VectorStore, PartialVectorStore]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        vector_store_id = kwargs["vector_store_id"]
        found = self._state.beta.vector_stores.get(vector_store_id)
        if not found:
            return httpx.Response(404)

        return httpx.Response(status_code=self._status_code, json=model_dict(found))

    @staticmethod
    def _build(partial: PartialVectorStore, request: httpx.Request) -> VectorStore:
        raise NotImplementedError


class VectorStoreUpdateRoute(StatefulRoute[VectorStore, PartialVectorStore]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        vector_store_id = kwargs["vector_store_id"]
        found = self._state.beta.vector_stores.get(vector_store_id)
        if not found:
            return httpx.Response(404)

        content: VectorStoreUpdateParams = json_loads(request.content)
        deserialized = model_dict(found)
        updated = model_parse(VectorStore, deserialized | content)
        return httpx.Response(status_code=self._status_code, json=model_dict(updated))

    @staticmethod
    def _build(partial: PartialVectorStore, request: httpx.Request) -> VectorStore:
        raise NotImplementedError


class VectorStoreDeleteRoute(
    StatefulRoute[VectorStoreDeleted, PartialVectorStoreDeleted]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.delete(
                url__regex=r"/vector_stores/(?P<vector_store_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route
        vector_store_id = kwargs["vector_store_id"]
        deleted = self._state.beta.vector_stores.delete(vector_store_id)
        return httpx.Response(
            status_code=200,
            json=model_dict(
                VectorStoreDeleted(
                    id=vector_store_id,
                    deleted=deleted,
                    object="vector_store.deleted",
                )
            ),
        )

    @staticmethod
    def _build(
        partial: PartialVectorStoreDeleted,
        request: httpx.Request,
    ) -> VectorStoreDeleted:
        raise NotImplementedError



from typing import Optional

import httpx

from openai.types.beta.assistant import Assistant

from ._base import _generic_builder
from ..._routes.assistants import AssistantCreateRoute
from ..._types.partials.assistants import PartialAssistant

__all__ = ["assistant_from_create_request"]


def assistant_from_create_request(
    request: httpx.Request,
    *,
    extra: Optional[PartialAssistant] = None,
) -> Assistant:
    return _generic_builder(AssistantCreateRoute, request, extra)



from typing import Optional

import httpx
from openai.types.beta.vector_store import VectorStore

from ._base import _generic_builder
from ..._routes.beta.vector_stores import VectorStoreCreateRoute
from ..._types.partials.vector_stores import PartialVectorStore

__all__ = ["vector_store_from_create_request"]


def vector_store_from_create_request(
    request: httpx.Request,
    *,
    extra: Optional[PartialVectorStore] = None,
) -> VectorStore:
    return _generic_builder(VectorStoreCreateRoute, request, extra)



from typing import List, Literal, TypedDict
from typing_extensions import NotRequired

__all__ = ["PartialFileObject", "PartialFileList", "PartialFileDeleted"]


class PartialFileObject(TypedDict):
    id: NotRequired[str]
    bytes: NotRequired[int]
    created_at: NotRequired[int]
    filename: NotRequired[str]
    object: NotRequired[Literal["file"]]
    purpose: NotRequired[
        Literal["fine-tune", "fine-tune-results", "assistants", "assistants_output"]
    ]
    status: NotRequired[Literal["uploaded", "processed", "error"]]
    status_details: NotRequired[str]


class PartialFileList(TypedDict):
    data: NotRequired[List[PartialFileObject]]
    object: NotRequired[Literal["list"]]


class PartialFileDeleted(TypedDict):
    id: NotRequired[str]
    object: NotRequired[Literal["file"]]
    deleted: NotRequired[bool]



import json
from typing import AsyncIterator, Generator, Generic, Optional, TypeVar, Union

from openai.types.beta import AssistantStreamEvent
from openai.types.chat import ChatCompletionChunk

from ._utils.aio import make_async_generator
from ._utils.serde import model_dict

__all__ = ["EventStream", "AsyncEventStream"]

M = TypeVar("M", bound=Union[AssistantStreamEvent, ChatCompletionChunk])


class BaseEventStream(Generic[M]):
    @staticmethod
    def _dump_event(event: M) -> tuple[Optional[bytes], Optional[bytes]]:
        if hasattr(event, "event") and hasattr(event, "data"):
            event_type: Optional[str] = getattr(event, "event", None)
            data: Optional[AssistantStreamEvent] = getattr(event, "data", None)
            if event_type is not None and data is not None:
                encoded_event = f"event: {event_type}\n".encode()
                encoded_data = f"data: {json.dumps(model_dict(data))}\n\n".encode()
                return encoded_event, encoded_data
        encoded_data = f"data: {json.dumps(model_dict(event))}\n\n".encode()
        return None, encoded_data

    def generate(self) -> Generator[M, None, None]:
        raise NotImplementedError


class EventStream(BaseEventStream[M]):
    """Event stream helper for building mock OpenAI server sent event stream"""

    def __iter__(self) -> Generator[bytes, None, None]:
        for _event in self.generate():
            t, d = self._dump_event(_event)
            if t:
                yield t
            if d:
                yield d

        yield b"event: done\n"
        yield b"data: [DONE]\n\n"


class AsyncEventStream(BaseEventStream[M]):
    """Async event stream helper for building mock OpenAI server sent event stream"""

    async def __aiter__(self) -> AsyncIterator[bytes]:
        async for _event in make_async_generator(self.generate()):
            t, d = self._dump_event(_event)
            if t:
                yield t
            if d:
                yield d
        yield b"event: done\n"
        yield b"data: [DONE]\n\n"



from typing import Any
from typing_extensions import override

import httpx
import respx

from openai.pagination import SyncCursorPage
from openai.types.beta.threads.runs.run_step import RunStep

from .._base import StatefulRoute

from ...stores import StateStore
from ..._types.partials.run_steps import PartialRunStep
from ..._types.partials.sync_cursor_page import PartialSyncCursorPage

from ..._utils.serde import model_dict


__all__ = ["RunStepListRoute", "RunStepRetrieveRoute"]


class RunStepListRoute(
    StatefulRoute[SyncCursorPage[RunStep], PartialSyncCursorPage[PartialRunStep]]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/runs/(?P<run_id>[a-zA-Z0-9\_]+)/steps"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        run_id = kwargs["run_id"]
        found_run = self._state.beta.threads.runs.get(run_id)
        if not found_run:
            return httpx.Response(404)

        limit = request.url.params.get("limit")
        order = request.url.params.get("order")
        after = request.url.params.get("after")
        before = request.url.params.get("before")

        data = self._state.beta.threads.runs.steps.list(
            thread_id,
            run_id,
            limit,
            order,
            after,
            before,
        )
        result_count = len(data)
        total_count = len(self._state.beta.threads.runs.steps.list(thread_id, run_id))
        has_data = bool(result_count)
        has_more = total_count != result_count
        first_id = data[0].id if has_data else None
        last_id = data[-1].id if has_data else None
        model = SyncCursorPage[RunStep](data=data)
        return httpx.Response(
            status_code=200,
            json=model_dict(model)
            | {"first_id": first_id, "last_id": last_id, "has_more": has_more},
        )

    @staticmethod
    def _build(
        partial: PartialSyncCursorPage[PartialRunStep],
        request: httpx.Request,
    ) -> SyncCursorPage[RunStep]:
        raise NotImplementedError


class RunStepRetrieveRoute(StatefulRoute[RunStep, PartialRunStep]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/runs/(?P<run_id>[a-zA-Z0-9\_]+)/steps/(?P<step_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        run_id = kwargs["run_id"]
        found_run = self._state.beta.threads.runs.get(run_id)
        if not found_run:
            return httpx.Response(404)

        step_id = kwargs["step_id"]
        found_run_step = self._state.beta.threads.runs.steps.get(step_id)
        if not found_run_step:
            return httpx.Response(404)

        return httpx.Response(status_code=200, json=model_dict(found_run_step))

    @staticmethod
    def _build(partial: PartialRunStep, request: httpx.Request) -> RunStep:
        raise NotImplementedError



from openai.types.embedding import Embedding
from openai.types.embedding_create_params import EmbeddingCreateParams
from openai.types.create_embedding_response import CreateEmbeddingResponse, Usage

import httpx
import respx

from ._base import StatelessRoute

from .._types.partials.embeddings import PartialCreateEmbeddingResponse

from .._utils.serde import json_loads, model_parse

__all__ = ["EmbeddingsCreateRoute"]


class EmbeddingsCreateRoute(
    StatelessRoute[
        CreateEmbeddingResponse,
        PartialCreateEmbeddingResponse,
    ]
):
    def __init__(self, router: respx.MockRouter) -> None:
        super().__init__(
            route=router.post(url__regex="/embeddings"),
            status_code=201,
        )

    @staticmethod
    def _build(
        partial: PartialCreateEmbeddingResponse,
        request: httpx.Request,
    ) -> CreateEmbeddingResponse:
        content: EmbeddingCreateParams = json_loads(request.content)
        embeddings = partial.get("data", [])
        response = CreateEmbeddingResponse(
            data=[model_parse(Embedding, e) for e in embeddings],
            model=partial.get("model", content["model"]),
            object="list",
            usage=model_parse(
                Usage,
                partial.get("usage", dict({"prompt_tokens": 0, "total_tokens": 0})),
            ),
        )
        return response



from typing import Optional

import httpx

from openai.types.beta.threads.message import Message

from ._base import _generic_builder
from ..._routes.beta.messages import MessageCreateRoute
from ..._types.partials.messages import PartialMessage

from ..._utils.faker import faker
from ..._utils.serde import model_parse
from ..._utils.time import utcnow_unix_timestamp_s

__all__ = ["message_from_create_request", "build_message"]


def message_from_create_request(
    thread_id: str,
    request: httpx.Request,
    *,
    extra: Optional[PartialMessage] = None,
) -> Message:
    partial: PartialMessage = {"thread_id": thread_id}
    if extra:
        partial |= extra
    return _generic_builder(MessageCreateRoute, request, partial)


def build_message(partial: PartialMessage) -> Message:
    default: PartialMessage = {
        "id": faker.beta.thread.message.id(),
        "created_at": utcnow_unix_timestamp_s(),
        "object": "thread.message",
    }
    return model_parse(Message, default | partial)



from openai.types.beta.assistant import Assistant

from ._base import _generic_merge_with_partial

from ..._types.partials.assistants import PartialAssistant

__all__ = ["merge_assistant_with_partial"]


def merge_assistant_with_partial(a: Assistant, p: PartialAssistant) -> Assistant:
    return _generic_merge_with_partial(Assistant, a, p)



import random

__all__ = ["faker"]


class Base62:
    BASE = 62
    CHARSET = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

    def encode(self, b: bytes) -> str:
        encoded: list[str] = []
        for byte in b:
            i = byte % self.BASE
            encoded.append(self.CHARSET[i])

        return "".join(reversed(encoded))


base62 = Base62()


def gen_id_suffix() -> str:
    b = random.randbytes(24)
    return base62.encode(b)


def gen_id(prefix: str, *, sep: str = "_") -> str:
    return prefix + sep + gen_id_suffix()


class AssistantProvider:
    def id(self) -> str:
        return gen_id("asst")


class ChatProvider:
    def __init__(self) -> None:
        self.completion = ChatProvider.CompletionProvider()

    class CompletionProvider:
        def id(self) -> str:
            return gen_id("chatcmpl")


class FileProvider:
    def id(self) -> str:
        return gen_id("file", sep="-")


class ThreadProvider:
    def __init__(self) -> None:
        self.message = ThreadProvider.MessageProvider()
        self.run = ThreadProvider.RunProvider()

    def id(self) -> str:
        return gen_id("thread")

    class MessageProvider:
        def id(self) -> str:
            return gen_id("msg")

    class RunProvider:
        def __init__(self) -> None:
            self.step = ThreadProvider.RunProvider.StepProvider()

        def id(self) -> str:
            return gen_id("run")

        class StepProvider:
            def __init__(self) -> None:
                self.step_details = (
                    ThreadProvider.RunProvider.StepProvider.StepDetailsProvider()
                )

            def id(self) -> str:
                return gen_id("step")

            class StepDetailsProvider:

                def __init__(self) -> None:
                    self.tool_call = (
                        ThreadProvider.RunProvider.StepProvider.StepDetailsProvider.ToolCallProvider()
                    )

                class ToolCallProvider:
                    def id(self) -> str:
                        return gen_id("call")


class VectorStoreProvider:
    def __init__(self) -> None:
        self.file_batch = VectorStoreProvider.VectorStoreFileBatchProvider()

    def id(self) -> str:
        return gen_id("vs")

    class VectorStoreFileBatchProvider:
        def id(self) -> str:
            return gen_id("vsfb")


class ModerationProvider:
    def id(self) -> str:
        return gen_id(prefix="modr", sep="-")


class Faker:
    def __init__(self) -> None:
        self.chat = ChatProvider()
        self.file = FileProvider()
        self.moderation = ModerationProvider()
        self.beta = Faker.BetaProviders()

    class BetaProviders:
        def __init__(self) -> None:
            self.assistant = AssistantProvider()
            self.thread = ThreadProvider()
            self.vector_store = VectorStoreProvider()


faker = Faker()



from typing import Annotated, Dict, List, Literal, TypedDict, Union
from typing_extensions import NotRequired

from openai._utils import PropertyInfo

__all__ = ["PartialRunStep"]


class PartialLastError(TypedDict):
    code: Literal["server_error", "rate_limit_exceeded"]
    message: str


class PartialMessageCreation(TypedDict):
    message_id: str


class PartialMessageCreationStepDetails(TypedDict):
    message_creation: PartialMessageCreation
    type: Literal["message_creation"]


class PartialCodeInterpreterOutputLogs(TypedDict):
    logs: str
    type: Literal["logs"]


class PartialCodeInterpreterOutputImageImage(TypedDict):
    file_id: str


class PartialCodeInterpreterOutputImage(TypedDict):
    image: PartialCodeInterpreterOutputImageImage
    type: Literal["image"]


class PartialCodeInterpreter(TypedDict):
    input: str
    outputs: List[
        Annotated[
            Union[PartialCodeInterpreterOutputLogs, PartialCodeInterpreterOutputImage],
            PropertyInfo(discriminator="type"),
        ]
    ]


class PartialCodeInterpreterToolCall(TypedDict):
    id: str
    code_interpreter: PartialCodeInterpreter
    type: Literal["code_interpreter"]


class PartialFileSearchToolCall(TypedDict):
    id: str
    file_search: object
    type: Literal["file_search"]


class PartialFunction(TypedDict):
    arguments: str
    name: str
    output: NotRequired[str]


class PartialFunctionToolCall(TypedDict):
    id: str
    function: PartialFunction
    type: Literal["function"]


class PartialToolCallsStepDetails(TypedDict):
    tool_calls: List[
        Annotated[
            Union[
                PartialCodeInterpreterToolCall,
                PartialFileSearchToolCall,
                PartialFunctionToolCall,
            ],
            PropertyInfo(discriminator="type"),
        ]
    ]
    type: Literal["tool_calls"]


class PartialUsage(TypedDict):
    completion_tokens: int
    prompt_tokens: int
    total_tokens: int


PartialStepDetails = Annotated[
    Union[PartialMessageCreationStepDetails, PartialToolCallsStepDetails],
    PropertyInfo(discriminator="type"),
]


class PartialRunStep(TypedDict):
    id: NotRequired[str]
    assistant_id: str
    cancelled_at: NotRequired[int]
    completed_at: NotRequired[int]
    created_at: NotRequired[int]
    expired_at: NotRequired[int]
    failed_at: NotRequired[int]
    last_error: NotRequired[PartialLastError]
    metadata: NotRequired[Dict[str, str]]
    object: NotRequired[Literal["thread.run.step"]]
    run_id: str
    status: Literal["in_progress", "cancelled", "failed", "completed", "expired"]
    step_details: PartialStepDetails
    thread_id: str
    type: Literal["message_creation", "tool_calls"]
    usage: NotRequired[PartialUsage]



from openai.types.beta.threads.runs.run_step import RunStep

from ..._types.partials.run_steps import PartialRunStep

from ..._utils.faker import faker
from ..._utils.serde import model_parse
from ..._utils.time import utcnow_unix_timestamp_s


__all__ = ["build_run_step"]


def build_run_step(partial: PartialRunStep) -> RunStep:
    return model_parse(
        RunStep,
        {
            "id": faker.beta.thread.run.step.id(),
            "created_at": utcnow_unix_timestamp_s(),
            "object": "thread.run.step",
        }
        | partial,
    )



from typing import Any
from typing_extensions import override

import httpx
import respx

from openai.pagination import SyncCursorPage
from openai.types.beta.threads.message import Message
from openai.types.beta.threads.message_deleted import MessageDeleted
from openai.types.beta.threads.message_update_params import MessageUpdateParams

from .._base import StatefulRoute

from ...stores import StateStore
from ..._types.partials.sync_cursor_page import PartialSyncCursorPage
from ..._types.partials.messages import PartialMessage, PartialMessageDeleted

from ..._utils.faker import faker
from ..._utils.serde import json_loads, model_dict, model_parse
from ..._utils.time import utcnow_unix_timestamp_s

__all__ = [
    "MessageCreateRoute",
    "MessageListRoute",
    "MessageRetrieveRoute",
    "MessageUpdateRoute",
    "MessageDeleteRoute",
]


class MessageCreateRoute(StatefulRoute[Message, PartialMessage]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/messages"
            ),
            status_code=201,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found = self._state.beta.threads.get(thread_id)
        if not found:
            return httpx.Response(404)

        model = self._build({"thread_id": thread_id}, request)
        self._state.beta.threads.messages.put(model)
        return httpx.Response(
            status_code=self._status_code,
            json=model_dict(model),
        )

    @staticmethod
    def _build(partial: PartialMessage, request: httpx.Request) -> Message:
        content = json_loads(request.content)
        defaults: PartialMessage = {
            "id": faker.beta.thread.message.id(),
            "content": [],
            "created_at": utcnow_unix_timestamp_s(),
            "object": "thread.message",
            "role": "user",
            "status": "completed",
        }
        if content.get("content"):
            value = content.get("content")
            if isinstance(value, str):
                defaults["content"].append(
                    {
                        "type": "text",
                        "text": {"annotations": [], "value": value},
                    }
                )
            else:
                for block in value:
                    if block.get("type") == "text":
                        defaults["content"].append(
                            {
                                "type": "text",
                                "text": {
                                    "annotations": [],
                                    "value": block.get("text"),
                                },
                            }
                        )

            del content["content"]

        return model_parse(Message, defaults | partial | content)


class MessageListRoute(
    StatefulRoute[SyncCursorPage[Message], PartialSyncCursorPage[PartialMessage]]
):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/messages"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found = self._state.beta.threads.get(thread_id)
        if not found:
            return httpx.Response(404)

        limit = request.url.params.get("limit")
        order = request.url.params.get("order")
        after = request.url.params.get("after")
        before = request.url.params.get("before")
        run_id = request.url.params.get("run_id")

        data = self._state.beta.threads.messages.list(
            thread_id,
            limit,
            order,
            after,
            before,
            run_id,
        )
        result_count = len(data)
        total_count = len(self._state.beta.threads.messages.list(thread_id))
        has_data = bool(result_count)
        has_more = total_count != result_count
        first_id = data[0].id if has_data else None
        last_id = data[-1].id if has_data else None
        model = SyncCursorPage[Message](data=data)
        return httpx.Response(
            status_code=200,
            json=model_dict(model)
            | {"first_id": first_id, "last_id": last_id, "has_more": has_more},
        )

    @staticmethod
    def _build(
        partial: PartialSyncCursorPage[PartialMessage],
        request: httpx.Request,
    ) -> SyncCursorPage[Message]:
        raise NotImplementedError


class MessageRetrieveRoute(StatefulRoute[Message, PartialMessage]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.get(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/messages/(?P<message_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        message_id = kwargs["message_id"]
        found_message = self._state.beta.threads.messages.get(message_id)
        if not found_message:
            return httpx.Response(404)

        return httpx.Response(status_code=200, json=model_dict(found_message))

    @staticmethod
    def _build(partial: PartialMessage, request: httpx.Request) -> Message:
        raise NotImplementedError


class MessageUpdateRoute(StatefulRoute[Message, PartialMessage]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.post(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/messages/(?P<message_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        message_id = kwargs["message_id"]
        found_message = self._state.beta.threads.messages.get(message_id)
        if not found_message:
            return httpx.Response(404)

        content: MessageUpdateParams = json_loads(request.content)
        deserialized = model_dict(found_message)
        updated = model_parse(Message, deserialized | content)
        self._state.beta.threads.messages.put(updated)

        return httpx.Response(status_code=200, json=model_dict(updated))

    @staticmethod
    def _build(partial: PartialMessage, request: httpx.Request) -> Message:
        raise NotImplementedError


class MessageDeleteRoute(StatefulRoute[MessageDeleted, PartialMessageDeleted]):
    def __init__(self, router: respx.MockRouter, state: StateStore) -> None:
        super().__init__(
            route=router.delete(
                url__regex=r"/threads/(?P<thread_id>[a-zA-Z0-9\_]+)/messages/(?P<message_id>[a-zA-Z0-9\_]+)"
            ),
            status_code=200,
            state=state,
        )

    @override
    def _handler(
        self,
        request: httpx.Request,
        route: respx.Route,
        **kwargs: Any,
    ) -> httpx.Response:
        self._route = route

        thread_id = kwargs["thread_id"]
        found_thread = self._state.beta.threads.get(thread_id)
        if not found_thread:
            return httpx.Response(404)

        message_id = kwargs["message_id"]
        deleted = self._state.beta.threads.messages.delete(message_id)
        return httpx.Response(
            status_code=200,
            json=model_dict(
                MessageDeleted(
                    id=message_id, deleted=deleted, object="thread.message.deleted"
                )
            ),
        )

    @staticmethod
    def _build(
        partial: PartialMessageDeleted,
        request: httpx.Request,
    ) -> MessageDeleted:
        raise NotImplementedError



import httpx
import respx

from openai.types.chat.chat_completion import ChatCompletion

from ._base import StatelessRoute

from .._types.partials.chat import PartialChatCompletion

from .._utils.faker import faker
from .._utils.serde import json_loads, model_parse
from .._utils.time import utcnow_unix_timestamp_s

__all__ = ["ChatCompletionsCreateRoute"]


class ChatCompletionsCreateRoute(StatelessRoute[ChatCompletion, PartialChatCompletion]):
    def __init__(self, router: respx.MockRouter) -> None:
        super().__init__(
            route=router.post(url__regex="/chat/completions"),
            status_code=201,
        )

    @staticmethod
    def _build(
        partial: PartialChatCompletion,
        request: httpx.Request,
    ) -> ChatCompletion:
        content = json_loads(request.content)
        defaults: PartialChatCompletion = {
            "id": partial.get("id", faker.chat.completion.id()),
            "created": partial.get("created", utcnow_unix_timestamp_s()),
            "object": "chat.completion",
        }
        return model_parse(ChatCompletion, defaults | partial | content)



from typing import Dict, Union


class ContentStore:
    def __init__(self) -> None:
        self._data: Dict[str, bytes] = {}

    def put(self, id: str, content: bytes) -> None:
        self._data[id] = content

    def get(self, id: str) -> Union[bytes, None]:
        return self._data.get(id)

    def delete(self, id: str) -> None:
        del self._data[id]



import pytest

from . import OpenAIMock


def pytest_configure(config: pytest.Config):
    config.addinivalue_line(
        "markers",
        "openai_mock: OpenAI API mocker object",
    )


@pytest.fixture()
def openai_mock() -> OpenAIMock:
    return OpenAIMock()
